unit series;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;
type
  PSerie = ^TSerie;
  TSerie = array of double;
  { TSeries }
  TSeries = class
  private
    FSeries: TList;
    function GetIdx(ALabel: string): Integer;
    function GetSerie(Header: string): TSerie;
    function GetLabels: TStrings;
  public
    Headers: TStrings;
    property Serie[Header: string]: TSerie read GetSerie; default;
    property Labels: TStrings read GetLabels;
    constructor Create;
    procedure Append(ASerie: string; ANumber: Double);
    procedure Append(ASerie: string; ALabel: string);
    destructor Destroy; override;
  end;

implementation

{ TSeries }

function TSeries.GetIdx(ALabel: string): Integer;
var
  i: Integer;
begin
  for i := 1 to Headers.Count - 1 do begin
     if Headers[i] = ALabel then begin
       Result := i;
       Exit;
     end;
  end;
  Result := 0;
end;

function TSeries.GetSerie(Header: string): TSerie;
var
  i : Integer;
begin
  Result := nil;
  i := GetIdx(Header);
  if i > 0 then Result := PSerie(FSeries[i])^;
end;

function TSeries.GetLabels: TStrings;
begin
  Result := TStrings(FSeries[0]);
end;

constructor TSeries.Create;
begin
  Headers := TStringList.Create;
  Headers.Add('t');
  FSeries := TList.Create;
  FSeries.Add(TStringList.Create);
end;

procedure TSeries.Append(ASerie: string; ANumber: Double);
var
  s: PSerie;
  i, len: Integer;
begin
  i := GetIdx(ASerie);
  if i < 1 then begin
    New(s);
    SetLength(s^, 1);
    s^[0] := ANumber;
    FSeries.Add(s);
    Headers.Add(ASerie);
  end else begin
    s := PSerie(FSeries[i]);
    len := Length(s^);
    SetLength(s^, len + 1);
    s^[len] := ANumber;
  end;
end;

procedure TSeries.Append(ASerie: string; ALabel: string);
begin
  Headers[0] := ASerie;
  TStrings(FSeries[0]).Append(ALabel);
end;

destructor TSeries.Destroy;
var
  i: Integer;
  s: PSerie;
begin
  TStrings(FSeries[0]).Free;
  for i := 1 to FSeries.Count - 1 do begin
    s := PSerie(FSeries[i]);
    SetLength(s^, 0);
    Dispose(s);
  end;
  FSeries.Free;
  Headers.Free;
end;

end.

