unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, Menus,
  ComCtrls, ActnList, StdCtrls, Buttons, lua52;

type

  { TfmMain }

  TfmMain = class(TForm)
    AppTerminate: TAction;
    edCmdline: TEdit;
    LoadData: TAction;
    ActionList1: TActionList;
    ImageList1: TImageList;
    mConsole: TMemo;
    MenuItem4: TMenuItem;
    miActions: TMenuItem;
    menuMain: TMainMenu;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    od: TOpenDialog;
    pc: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    procedure AppTerminateExecute(Sender: TObject);
    procedure edCmdlineEnter(Sender: TObject);
    procedure edCmdlineExit(Sender: TObject);
    procedure edCmdlineKeyPress(Sender: TObject; var Key: char);
    procedure FormCreate(Sender: TObject);
    procedure LoadDataExecute(Sender: TObject);
    procedure AddAction(cmd: string);
    procedure miActionsExec(Sender: TObject);
  private
    procedure Error(msg: string);
    procedure LoadConfig;
    procedure DoAction(cmd: string);
    procedure CenterForm;
  public
    { public declarations }
  end;

var
  fmMain: TfmMain;

implementation

uses luaintf;

{$R *.lfm}

{ TfmMain }

procedure TfmMain.LoadDataExecute(Sender: TObject);
var
  sl: TStrings;
begin
  if od.Execute then begin
    sl := TStringList.Create;
    sl.LoadFromFile(od.FileName);
    sl.Free;
  end;
end;

procedure TfmMain.AddAction(cmd: string);
var
  i : Integer;
  mi : TMenuItem;
begin
  for i := 0 to miActions.Count - 1 do begin
    if cmd = miActions.Items[i].Caption then Exit;
  end;
  mi := TMenuItem.Create(Self);
  mi.Caption := cmd;
  mi.OnClick := @miActionsExec;
  miActions.Add(mi);
end;

procedure TfmMain.miActionsExec(Sender: TObject);
begin
  DoAction(TMenuItem(Sender).Caption);
end;

procedure TfmMain.Error(msg: string);
begin
  ShowMessage(msg);
end;

procedure TfmMain.LoadConfig;
var
  i: Integer;
  dir, item: string;
  sr: TSearchRec;
begin
  if 0 <> luaL_dofile(LUA, PChar(ChangeFileExt(ParamStr(0), '.cfg'))) then Exit;
  lua_getglobal(LUA, 'ext_dir');
  if not lua_isstring(LUA, -1) then Exit;
  dir := lua_tostring(LUA, -1);
  lua_pop(LUA, 1);
  if not DirectoryExists(dir) then begin
    Error('CFGERR: specified library directory does not exist');
    Exit;
  end;
  dir := IncludeTrailingPathDelimiter(dir);
  if FindFirst(dir + '*.lua', 0, sr) = 0 then begin
     repeat
       if luaL_dofile(LUA, PChar(dir + sr.Name)) <> 0 then
           Error('CFGERR: loading failed - ' + sr.Name);
     until FindNext(sr) <> 0;
     FindClose(sr);
  end;
  lua_getglobal(LUA, 'shortcuts');
  if not lua_istable(LUA, -1) then begin
    Error('CFGERR: shortcuts must be specified as lua table');
    Exit;
  end;
  i := 0;
  while true do begin
    Inc(i);
    lua_rawgeti(LUA, -1, i);
    if not lua_isstring(LUA, -1) then Break;
    item := lua_tostring(LUA, -1);
    lua_pop(LUA, 1);
    lua_getglobal(LUA, PChar(item));
    if lua_type(LUA, -1) = LUA_TFUNCTION then begin
      AddAction(item);
    end else begin
      Error('CFGERR: no function defined for shortcut - ' + item);
    end;
    lua_pop(LUA, 1);
  end;
end;

procedure TfmMain.DoAction(cmd: string);
begin
  pc.TabIndex:= 1;
  mConsole.Text := 'Command received: ' + cmd;
end;

procedure TfmMain.CenterForm;
begin
  Left := (Screen.Width - Width) div 2;
  if Left < 0 then Left := 0;
  Top := (Screen.Height - Height) div 2;
  if Top < 0 then Top := 0;
end;

procedure TfmMain.AppTerminateExecute(Sender: TObject);
begin
  Application.Terminate;
end;

procedure TfmMain.edCmdlineEnter(Sender: TObject);
begin
  with edCmdline do begin
    Font.Color := clBlack;
    Text := '';
  end;
end;

procedure TfmMain.edCmdlineExit(Sender: TObject);
begin
  with edCmdline do begin
    Font.Color := clMedGray;
    Text := 'Enter command here (type ''help'' for instructions)';
  end;

end;

procedure TfmMain.edCmdlineKeyPress(Sender: TObject; var Key: char);
begin
  if Key = #13 then DoAction(Trim(edCmdline.Text));
end;

procedure TfmMain.FormCreate(Sender: TObject);
begin
  LoadConfig;
  CenterForm;
end;

end.

